#!/usr/bin/perl -w
#
# $Id: mailmsg_in.pl,v 1.1 2002/12/18 01:25:47 mpeppler Exp $

use strict;

use Sybase::CTlib;

my $props = { CON_PROPS => { CS_SYB_CHARSET => 'utf8'}};

my $dbh = new Sybase::CTlib 'sa', 'secret', 'GENERALZOD', $0, $props;
$dbh->ct_sql("use payment_db");
$dbh->ct_sql("set textsize 1500000");

open(IN, "MailMessage.bcp") || die "Can't open bcp file: $!";
local $/ = "(**)\n";

my @cols = qw(accountId	charset	contentType fromAddress	isForwarded isReceived
	      lostDataError mailMessageID messageID mimeVersion receivedDate
	      recipient	replyTo	sentDate subject text transferEncoding
	      xMailer xSender scrubbed forwardingUserId	forwardingDate
	      storedOn bcc cc isReplyFromKagi);

my %noquote = (accountId => 1, isForwarded => 1, isReceived => 1, 
	       lostDataError => 1, mailMessageID => 1, forwardingUserId => 1,
	       isReplyFromKagi => 1);

my %type = ( accountId => [CS_NUMERIC_TYPE, 36, 9],
	     isForwarded => [CS_INT_TYPE],
	     isReceived => [CS_INT_TYPE],
	     lostDataError => [CS_INT_TYPE],
	     mailMessageID => [CS_NUMERIC_TYPE, 36, 9],
	     forwardingUserId => [CS_NUMERIC_TYPE, 36, 0],
	     isReplyFromKagi => [CS_INT_TYPE] );

createProc($dbh);

my %row;
my $ret;
while(<IN>) {
    chomp;

    @row{@cols} = split(/!~~!/, $_);

    if(($ret = insertProcRow($dbh, \%row)) < 0) {
	warn "insert failed ($ret) for $_";
	next;
    }

    if(0) {
	my $sql = "insert MailMessage( ". join(',', @cols) .") values (";

	foreach my $c (@cols) {
	    if($c eq 'text') {
		$sql .= '" ",';
		next;
	    }
	    if(!defined($row{$c}) || length($row{$c}) == 0) {
		$sql .= 'NULL';
	    } else {
		if($noquote{$c}) {
		    $sql .= $row{$c};
		} else {
		    $row{$c} =~ s/\"/\"\"/g;
		    $sql .= "\"$row{$c}\"";
		}
	    }
	    $sql .= ',';
	}
	$sql =~ s/,$/\)/;
	eval {
	    insertRow($dbh, $sql);
	}; 
	if($@) {
	    warn "$@\n";
	    next;
	}
	#    $dbh->ct_sql($sql);
    }

    if($row{text}) {
	updateText($dbh, $row{mailMessageID}, $row{text});
    }
}

sub insertRow {
    my $dbh = shift;
    my $sql = shift;

    my $restype;
    my $ret;
    my $fail_flag = 0;
    $dbh->ct_execute($sql) == CS_SUCCEED || die "Can't exec $sql";
    while(($ret = $dbh->ct_results($restype)) == CS_SUCCEED) {
	if($restype == CS_CMD_FAIL) {
	    ++$fail_flag;
	}
	next unless $dbh->ct_fetchable($restype);
	while(my @d = $dbh->ct_fetch) {
	    print "@d\n";
	}
    }
    if($fail_flag || $ret == CS_FAIL) {
	die "ct_results failed for $sql";
    }
}

sub updateText {
    my ($dbh, $key, $data) = @_;

    my $restype;
    my @d;
    $dbh->ct_execute("select text from MailMessage where mailMessageID = $key");
    while($dbh->ct_results($restype) == CS_SUCCEED) {
	next unless $dbh->ct_fetchable($restype);
	while(@d = $dbh->ct_fetch) {
	    $dbh->ct_data_info(CS_GET, 1);
	}
    }
    $dbh->ct_command(CS_SEND_DATA_CMD, '', CS_UNUSED, CS_COLUMN_DATA);
    $dbh->ct_data_info(CS_SET, 1, {total_txtlen => length($data)});
    $dbh->ct_send_data($data, length($data));
    $dbh->ct_send;
    while($dbh->ct_results($restype) == CS_SUCCEED) {
	next unless $dbh->ct_fetchable($restype);
	while(@d = $dbh->ct_fetch) {
#	    print "@d\n";
	}
    }
}

sub createProc {
    my $dbh = shift;

    $dbh->ct_sql("drop proc mp_ins_mailmessage");
    $dbh->ct_sql(<<'EOF');
create proc mp_ins_mailmessage
(
	@accountId        numeric(36,9)  
,	@charset          varchar(30)               = null
,	@contentType      varchar(255)        = null
,	@fromAddress      varchar(255)        = null
,	@isForwarded      int        = null
,	@isReceived       int        = null
,	@lostDataError    int        = null
,	@mailMessageID    numeric(36,9)
,	@messageID        varchar(255)        = null
,	@mimeVersion      varchar(30)        = null
,	@receivedDate     datetime        = null
,	@recipient        varchar(255)        = null
,	@replyTo          varchar(255)        = null
,	@sentDate         datetime        = null
,	@subject          varchar(255)        = null
--	text             text              NULL,
,	@transferEncoding varchar(30)        = null
,	@xMailer          varchar(60)        = null
,	@xSender          varchar(60)        = null
,	@scrubbed         varchar(3)
,	@forwardingUserId numeric(36,0)        = null
,	@forwardingDate   datetime        = null
,	@storedOn         datetime
,	@bcc              varchar(255)        = null
,	@cc               varchar(255)        = null
,	@isReplyFromKagi  int        = null)
as
insert MailMessage( accountId, charset, contentType, fromAddress,
     isForwarded, isReceived, lostDataError, mailMessageID, messageID, 
     mimeVersion, receivedDate, recipient, replyTo, sentDate, subject, text,
     transferEncoding, xMailer, xSender, scrubbed, forwardingUserId,
     forwardingDate, storedOn, bcc, cc, isReplyFromKagi)
values(@accountId
,	@charset     
,	@contentType 
,	@fromAddress 
,	@isForwarded 
,	@isReceived  
,	@lostDataError 
,	@mailMessageID 
,	@messageID     
,	@mimeVersion   
,	@receivedDate  
,	@recipient     
,	@replyTo       
,	@sentDate      
,	@subject       
, ' ' --	text             text              NULL,
,	@transferEncoding
,	@xMailer         
,	@xSender         
,	@scrubbed        
,	@forwardingUserId
,	@forwardingDate  
,	@storedOn        
,	@bcc             
,	@cc              
,	@isReplyFromKagi)
EOF


}

sub insertProcRow {
    my $dbh = shift;
    my $row = shift;

    $dbh->ct_command(CS_RPC_CMD, "mp_ins_mailmessage", CS_NULLTERM, CS_NO_RECOMPILE) == CS_SUCCEED || return -1;

    my %prm;

    foreach my $c (grep($_ ne 'text', @cols)) {
	$prm{name} = "\@$c";
	if($type{$c}) {
	    $prm{datatype} = $type{$c}->[0];
	} else {
	    $prm{datatype} = CS_CHAR_TYPE;
	}
	$prm{status} = CS_INPUTVALUE;
	$prm{value} = $row->{$c};
	if(!defined($row->{$c}) || length($row->{$c}) == 0) {
	    $prm{indicator} = -1;
	} else {
	    $prm{indicator} = CS_UNUSED;
	}
	$dbh->ct_param(\%prm) == CS_SUCCEED || return -2;
    }
    $dbh->ct_send == CS_SUCCEED || return -3;
    my $restype;
    my $ret;
    my $fail_flag = 0;
    my $status;
    while(($ret = $dbh->ct_results($restype)) == CS_SUCCEED) {
	if($restype == CS_CMD_FAIL) {
	    ++$fail_flag;
	}
	next unless $dbh->ct_fetchable($restype);
	while(my @d = $dbh->ct_fetch) {
	    if($restype == CS_STATUS_RESULT) {
		$status = $d[0];
	    } else {
		print "@d\n";
	    }
	}
    }
    if($ret == CS_FAIL || $fail_flag) {
	return -4;
    }
    if($status != 0) {
	return -5;
    }
    return 0;
}
